@@ -264,18 +264,28 @@ generate_c_compiled_deriv_dde <- function(dat) {
 generate_c_compiled_rhs_r <- function(dat, rewrite) {
   discrete <- dat$features$discrete
   if (discrete) {
-    time_access <- "INTEGER"
+    time_access <- "scalar_int"
     time_type <- "int"
   } else {
-    time_access <- "REAL"
+    time_access <- "scalar_real"
     time_type <- "double"
   }
+
+  args <- c(SEXP = dat$meta$c$ptr, SEXP = dat$meta$time, SEXP = dat$meta$state)
+
+  ## Early exit in the case of delay models:
+  if (dat$features$has_delay && dat$features$discrete) {
+    body <- c('Rf_error("Can\'t call update() on delay models");',
+              "return R_NilValue;")
+    return(c_function("SEXP", dat$meta$c$rhs_r, args, body))
+  }
+
   body <- collector()
   body$add("SEXP %s = PROTECT(allocVector(REALSXP, LENGTH(%s)));",
-          dat$meta$result, dat$meta$state)
+           dat$meta$result, dat$meta$state)
   body$add("%s *%s = %s(%s, 1);",
-          dat$meta$c$internal_t, dat$meta$internal,
-          dat$meta$c$get_internal, dat$meta$c$ptr)
+           dat$meta$c$internal_t, dat$meta$internal,
+           dat$meta$c$get_internal, dat$meta$c$ptr)
   if (dat$features$has_output) {
     output_ptr <- sprintf("%s_ptr", dat$meta$output)
     body$add("SEXP %s = PROTECT(allocVector(REALSXP, %s));",
@@ -293,9 +303,9 @@ generate_c_compiled_rhs_r <- function(dat, rewrite) {
   }
 
   eval_rhs <- sprintf_safe(
-    "%s(%s, %s(%s)[0], REAL(%s), REAL(%s), %s);",
+    '%s(%s, %s(%s, "%s"), REAL(%s), REAL(%s), %s);',
     dat$meta$c$rhs, dat$meta$internal, time_access, dat$meta$time,
-    dat$meta$state, dat$meta$result, dat$meta$output)
+    dat$meta$time, dat$meta$state, dat$meta$result, dat$meta$output)
 
   ## In order to run the derivative calculation safely, we have to set
   ## the initial time.  But in order to make this safe, we need to put
@@ -309,8 +319,9 @@ generate_c_compiled_rhs_r <- function(dat, rewrite) {
       sprintf_safe("const %s %s = %s;",
                    time_type, dat$meta$initial_time, initial_time),
       c_expr_if(sprintf_safe("ISNA(%s)", dat$meta$initial_time),
-                sprintf_safe("%s = %s(%s)[0];",
-                             initial_time, time_access, dat$meta$time)))
+                sprintf_safe('%s = %s(%s, "%s");',
+                             initial_time, time_access, dat$meta$time,
+                             dat$meta$time)))
     reset_initial_time <-
       c_expr_if(sprintf_safe("ISNA(%s)", dat$meta$initial_time),
                 sprintf_safe("%s = %s;", initial_time, dat$meta$initial_time))
@@ -326,7 +337,6 @@ generate_c_compiled_rhs_r <- function(dat, rewrite) {
   body$add("UNPROTECT(1);")
   body$add("return %s;", dat$meta$result)
 
-  args <- c(SEXP = dat$meta$c$ptr, SEXP = dat$meta$time, SEXP = dat$meta$state)
   c_function("SEXP", dat$meta$c$rhs_r, args, body$get())
 }
 
@@ -449,13 +459,16 @@ generate_c_compiled_set_user <- function(eqs, dat) {
 
 
 generate_c_compiled_set_initial <- function(dat, rewrite) {
-  if (!dat$features$has_delay) {
-    return(NULL)
-  }
-
   set_initial1 <- function(x) {
     rhs <- c_extract_variable(x, dat$data$elements, dat$meta$state, rewrite)
-    sprintf_safe("%s = %s;", rewrite(x$initial), rhs)
+    if (dat$data$elements[[x$name]]$rank == 0) {
+      sprintf_safe("%s = %s;", rewrite(x$initial), rhs)
+    } else {
+      el <- dat$data$elements[[x$name]]
+      sprintf_safe("memcpy(%s, %s, %s * sizeof(%s));",
+                   rewrite(x$initial), rhs, rewrite(el$dimnames$length),
+                   el$storage_type)
+    }
   }
 
   ## TODO: see generate_c_compiled_initial_conditions for more that
@@ -477,17 +490,19 @@ generate_c_compiled_set_initial <- function(dat, rewrite) {
   }
 
   body <- collector()
-  body$add("%s *%s = %s(%s, 1);",
-           internal_t, internal, dat$meta$c$get_internal, ptr)
-  body$add("const double %s = REAL(%s)[0];", dat$meta$time, time_ptr)
-  body$add("%s = %s;", rewrite(dat$meta$initial_time), dat$meta$time)
-  if (!dat$features$discrete) {
-    body$add("%s = INTEGER(%s)[0];", rewrite(dat$meta$c$use_dde), use_dde_ptr)
+  if (dat$features$has_delay) {
+    body$add("%s *%s = %s(%s, 1);",
+             internal_t, internal, dat$meta$c$get_internal, ptr)
+    body$add("const double %s = REAL(%s)[0];", dat$meta$time, time_ptr)
+    body$add("%s = %s;", rewrite(dat$meta$initial_time), dat$meta$time)
+    if (!dat$features$discrete) {
+      body$add("%s = INTEGER(%s)[0];", rewrite(dat$meta$c$use_dde), use_dde_ptr)
+    }
+    body$add("if (%s != R_NilValue) {", state_ptr)
+    body$add("  double * %s = REAL(%s);", dat$meta$state, state_ptr)
+    body$add(paste0("  ", set_initial_variables))
+    body$add("}")
   }
-  body$add("if (%s != R_NilValue) {", state_ptr)
-  body$add("  double * %s = REAL(%s);", dat$meta$state, state_ptr)
-  body$add(paste0("  ", set_initial_variables))
-  body$add("}")
   body$add("return R_NilValue;")
 
   c_function("SEXP", dat$meta$c$set_initial, args, body$get())
@@ -526,13 +541,14 @@ generate_c_compiled_initial_conditions <- function(dat, rewrite) {
   body <- collector()
   if (dat$features$initial_time_dependent) {
     if (dat$features$discrete) {
-      type <- "int"
-      unpack <- "INTEGER"
+      time_type <- "int"
+      time_access <- "scalar_int"
     } else {
-      type <- "double"
-      unpack <- "REAL"
+      time_type <- "double"
+      time_access <- "scalar_real"
     }
-    body$add("%s %s = %s(%s)[0];", type, dat$meta$time, unpack, time_ptr)
+    body$add('%s %s = %s(%s, "%s");',
+             time_type, dat$meta$time, time_access, time_ptr, dat$meta$time)
   }
   body$add("%s *%s = %s(%s, 1);",
           dat$meta$c$internal_t, dat$meta$internal,
@@ -563,12 +579,16 @@ generate_c_compiled_metadata <- function(dat, rewrite) {
       sprintf_safe("SET_VECTOR_ELT(%s, %d, ScalarInteger(%s));",
                    target, i - 1L, rewrite(d$dimnames$length))
     } else {
+      ## NOTE: need to use array_dim_name here because we might have
+      ## removed the dimension variable. However, this exists only
+      ## through a short scope here and we could really use anything.
+      name <- array_dim_name(d$name)
       c(sprintf_safe("SET_VECTOR_ELT(%s, %d, allocVector(INTSXP, %d));",
                      target, i - 1L, d$rank),
         sprintf_safe("int * %s = INTEGER(VECTOR_ELT(%s, %d));",
-                     d$dimnames$length, target, i - 1L),
+                     name, target, i - 1L),
         sprintf_safe("%s[%d] = %s;",
-                     d$dimnames$length, seq_len(d$rank) - 1L,
+                     name, seq_len(d$rank) - 1L,
                      vcapply(d$dimnames$dim, rewrite, USE.NAMES = FALSE)))
     }
   }
@@ -703,6 +723,11 @@ generate_c_compiled_library <- function(dat, is_package) {
   if ("sum" %in% used) {
     v <- c(v, "odin_sum1", "odin_isum1")
   }
+  if (dat$features$discrete) {
+    v <- c(v, "scalar_int")
+  } else {
+    v <- c(v, "scalar_real")
+  }
 
   if ("odin_sum" %in% used) {
     ## We can do better than this but it requires going through a lot
@@ -743,12 +768,20 @@ generate_c_compiled_library <- function(dat, is_package) {
 }
 
 
-generate_c_compiled_include <- function(dat, is_package) {
-  include <- dat$config$include
-  if (is_package) {
-    include
-  } else {
-    list(declaration = c_flatten_eqs(lapply(include, "[[", "declaration")),
-         definition = c_flatten_eqs(lapply(include, "[[", "definition")))
+generate_c_compiled_include <- function(dat) {
+  include <- dat$config$include$data
+
+  if (length(include) == 0) {
+    return(NULL)
   }
+
+  ## When this has been through the serialise/deserialise step it has
+  ## lost some structure, so we return named vectors
+  ret <- lapply(include, function(x) {
+    nms <- list_to_character(x$names)
+    list(names = nms,
+         declarations = set_names(list_to_character(x$declarations), nms),
+         definitions = set_names(list_to_character(x$definitions), nms))
+  })
+  unlist(ret, FALSE)
 }
